package com.hz.pm.api.projectlib.handle;

import com.hz.pm.api.projectlib.model.entity.Project;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import lombok.AllArgsConstructor;
import lombok.Getter;
import org.springframework.core.Ordered;

import java.util.List;

/**
 * 流程业务抽象处理类
 *
 * @author CMM
 * @since 2023/02/24 14:22
 */

public abstract class AbstractProcessBusinessHandle implements Ordered {
    /**
     * 构造，子类必须实现
     */
    protected AbstractProcessBusinessHandle() {

    }

    /**
     * 抽象的，所有具体处理者应该实现的处理逻辑
     *
     * @param project         \
     * @param processSchedule \
     */
    abstract void businessHandle(Project project, List<ProcessDetailVO> processSchedule);

    abstract ProcessNode getProcessNode();

    @Getter
    @AllArgsConstructor
    enum ProcessNode {

        PROJECT_DECLARED("项目申报", 1),
        PROJECT_REVIEW("项目评审", 2),
        APPROVAL_AMOUNT("资金下达", 3),
        PROJECT_APPROVAL("项目立项", 4),
        PROJECT_PURCHASE("招标采购", 5),
        PROJECT_ADAPTION("适配改造", 6),
        SYSTEM_SELF_TEST("系统自测", 7),
        SYSTEM_TEST_VALID("测试验证", 8),
        FIRST_INSPECTED("项目初验", 9),
        PILOT_RUNNING("上线试运行", 10),
        XCFHX_REVIEW("信创测评", 11),
        FINAL_INSPECTED("项目终验", 12);


        private final String name;
        private final Integer order;

    }

}
