package com.hz.pm.api.projectlib.handle;

import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.entity.PurchaseInst;
import com.hz.pm.api.projectdeclared.service.IPurchaseInstService;
import com.hz.pm.api.projectdeclared.service.IPurchaseService;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.enumeration.InstTypeEnum;
import com.hz.pm.api.projectlib.model.enumeration.TenderStatusEnum;
import com.hz.pm.api.todocenter.utils.BuildUserUtils;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.bean.vo.ProcessInfoVO;
import com.wflow.workflow.bean.vo.ProcessProgressVo;
import com.wflow.workflow.enums.StepStatusEnum;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * 项目终验处理
 *
 * @author CMM
 * @since 2023/02/24 14:35
 */
@Component
@RequiredArgsConstructor
public class FinalInspectionHandle extends AbstractProcessBusinessHandle {

    private final IPurchaseInstService purchaseInstService;
    private final IPurchaseService purchaseService;
    private final ProcessInstanceService processInstanceService;
    private final BuildUserUtils buildUserUtils;

    @Override
    ProcessNode getProcessNode() {
        return ProcessNode.FINAL_INSPECTED;
    }

    @Override
    void businessHandle(Project project, List<ProcessDetailVO> processSchedule) {
        List<Purchase> purchases = purchaseService.listByProjectId(project.getId());
        Map<Long, Purchase> purchaseMap = purchases.stream()
                .filter(w -> TenderStatusEnum.FINALLY_INSPECTED_FAILED.eq(w.getStatus())
                        || TenderStatusEnum.FINALLY_INSPECTED_PASSED.eq(w.getStatus()))
                .collect(Collectors.toMap(Purchase::getId, Function.identity()));
        if (purchaseMap.isEmpty()) {
            return;
        }
        ProcessDetailVO detail = new ProcessDetailVO();
        detail.setProcessName(getProcessNode().getName());
        detail.setStepStatus(StepStatusEnum.COMPLETED);
        detail.setProcessInfoList(new ArrayList<>());
        processSchedule.add(detail);
        Map<Long, PurchaseInst> instMap = purchaseInstService.list(purchaseMap.keySet(), InstTypeEnum.PROJECT_FINAL_INSPECTION.getCode());
        for (Map.Entry<Long, PurchaseInst> entry : instMap.entrySet()) {
            if (detail.getFinishTime() == null || detail.getFinishTime().isBefore(entry.getValue().getCreateOn())) {
                detail.setFinishTime(entry.getValue().getCreateOn());
            }
            ProcessProgressVo instanceDetail = processInstanceService.getProgressInstanceDetail(null, entry.getValue().getInstCode());
            buildUserUtils.buildUserByProcessInfo(instanceDetail.getProgressInfo());
            ProcessInfoVO tenderProgress = new ProcessInfoVO();
            tenderProgress.setProcessName(purchaseMap.get(entry.getKey()).getBidName());
            tenderProgress.setProcessProgressVo(instanceDetail);
            detail.getProcessInfoList().add(tenderProgress);
        }
    }

    @Override
    public int getOrder() {
        return getProcessNode().getOrder();
    }

}
