package com.hz.pm.api.projectlib.handle;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.service.IProjectStatusChangeService;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.enums.StepStatusEnum;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 初验处理
 *
 * @author CMM
 * @since 2023/02/27 10:09
 */
@Component
@RequiredArgsConstructor
public class FirstInspectedHandle extends AbstractProcessBusinessHandle {

    private final IProjectStatusChangeService projectStatusChangeService;

    @Override
    ProcessNode getProcessNode() {
        return ProcessNode.FIRST_INSPECTED;
    }

    @Override
    public int getOrder() {
        return getProcessNode().getOrder();
    }

    @Override
    void businessHandle(Project project, List<ProcessDetailVO> processSchedule) {
        ProcessDetailVO detail = new ProcessDetailVO();
        detail.setProcessName(getProcessNode().getName());
        processSchedule.add(detail);
        Wrapper<ProjectStatusChange> query = Wrappers.lambdaQuery(ProjectStatusChange.class)
                .eq(ProjectStatusChange::getProjectCode, project.getProjectCode())
                .eq(ProjectStatusChange::getEvent, ProjectStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES)
                .orderByDesc(ProjectStatusChange::getCreateOn)
                .last(BizConst.LIMIT_1);
        ProjectStatusChange change = projectStatusChangeService.getOne(query);
        if (change != null) {
            detail.setStepStatus(StepStatusEnum.COMPLETED);
            detail.setFinishTime(change.getCreateOn());
        } else {
            detail.setStepStatus(StepStatusEnum.NOT_START);
        }
    }

}
