package com.hz.pm.api.projectlib.handle;

import com.hz.pm.api.projectlib.model.entity.Project;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.enums.StepStatusEnum;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

/**
 * 流程执行链
 *
 * @author CMM
 * @since 2023/02/26 21:36
 */
@Component
@RequiredArgsConstructor
public class ProcessProgressContext {

    /**
     * 具体处理者的集合
     */
    private final List<AbstractProcessBusinessHandle> processBusinessHandles;

    public List<ProcessDetailVO> handle(Project project, List<ProcessDetailVO> processSchedule) {
        if (!processBusinessHandles.isEmpty()) {
            processBusinessHandles.sort(Comparator.comparing(AbstractProcessBusinessHandle::getOrder));
            for (AbstractProcessBusinessHandle processBusinessHandle : processBusinessHandles) {
                processBusinessHandle.businessHandle(project, processSchedule);
            }
            for (int i = processSchedule.size() - 1; i >= 1; i--) {
                ProcessDetailVO curr = processSchedule.get(i);
                if (curr.getStepStatus() == null ||
                        curr.getStepStatus().equals(StepStatusEnum.NOT_START)) {
                    continue;
                }
                ProcessDetailVO pre = processSchedule.get(i - 1);
                pre.setStepStatus(StepStatusEnum.COMPLETED);
            }
            return processSchedule;
        }
        return Collections.emptyList();
    }
}
