package com.hz.pm.api.projectlib.handle;

import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.entity.PurchaseInst;
import com.hz.pm.api.projectdeclared.service.IPurchaseInstService;
import com.hz.pm.api.projectdeclared.service.IPurchaseService;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.enumeration.InstTypeEnum;
import com.hz.pm.api.projectlib.model.enumeration.TenderAdaptStatusEnum;
import com.hz.pm.api.todocenter.utils.BuildUserUtils;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.bean.vo.ProcessInfoVO;
import com.wflow.workflow.bean.vo.ProcessProgressVo;
import com.wflow.workflow.enums.StepStatusEnum;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * AdaptionHandle
 * </p>
 *
 * @author WendyYang
 * @since 09:29 2024/4/9
 */
@Component
@RequiredArgsConstructor
public class ProjectAdaptionHandle extends AbstractProcessBusinessHandle {

    private final IPurchaseService purchaseService;
    private final IPurchaseInstService purchaseInstService;
    private final BuildUserUtils buildUserUtils;
    private final ProcessInstanceService processInstanceService;

    @Override
    void businessHandle(Project project, List<ProcessDetailVO> processSchedule) {
        List<Purchase> purchases = purchaseService.listByProjectId(project.getId());
        Map<Long, Purchase> purchaseMap = purchases.stream()
                .filter(w -> TenderAdaptStatusEnum.ADAPT_INFO_PASSED.eq(w.getAdaptStatus())
                        || TenderAdaptStatusEnum.ADAPT_INFO_FAILED.eq(w.getAdaptStatus()))
                .collect(Collectors.toMap(Purchase::getId, Function.identity()));
        if (purchaseMap.isEmpty()) {
            return;
        }
        ProcessDetailVO detail = new ProcessDetailVO();
        detail.setProcessName(getProcessNode().getName());
        detail.setStepStatus(StepStatusEnum.COMPLETED);
        detail.setProcessInfoList(new ArrayList<>());
        processSchedule.add(detail);
        Map<Long, PurchaseInst> instMap = purchaseInstService.list(purchaseMap.keySet(), InstTypeEnum.ADAPTION.getCode());
        for (Map.Entry<Long, PurchaseInst> entry : instMap.entrySet()) {
            if (detail.getFinishTime() == null || detail.getFinishTime().isBefore(entry.getValue().getCreateOn())) {
                detail.setFinishTime(entry.getValue().getCreateOn());
            }
            ProcessProgressVo instanceDetail = processInstanceService.getProgressInstanceDetail(null, entry.getValue().getInstCode());
            buildUserUtils.buildUserByProcessInfo(instanceDetail.getProgressInfo());
            ProcessInfoVO tenderProgress = new ProcessInfoVO();
            tenderProgress.setProcessName(purchaseMap.get(entry.getKey()).getBidName());
            tenderProgress.setProcessProgressVo(instanceDetail);
            detail.getProcessInfoList().add(tenderProgress);
        }
    }

    @Override
    ProcessNode getProcessNode() {
        return ProcessNode.PROJECT_ADAPTION;
    }

    @Override
    public int getOrder() {
        return getProcessNode().getOrder();
    }

}
