package com.hz.pm.api.projectlib.handle;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.projectlib.service.IProjectStatusChangeService;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.enums.StepStatusEnum;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.List;

import static com.hz.pm.api.projectlib.model.enumeration.ProjectStatusEnum.ON_PURCHASING;
import static com.hz.pm.api.projectlib.model.enumeration.ProjectStatusEnum.TO_BE_PURCHASED;

/**
 * 招标采购处理
 *
 * @author CMM
 * @since 2023/02/24 14:35
 */
@Component
@RequiredArgsConstructor
public class ProjectPurchaseHandle extends AbstractProcessBusinessHandle {

    private final IProjectStatusChangeService projectStatusChangeService;
    private final IProjectService projectService;

    @Override
    public int getOrder() {
        return getProcessNode().getOrder();
    }

    @Override
    ProcessNode getProcessNode() {
        return ProcessNode.PROJECT_PURCHASE;
    }

    @Override
    void businessHandle(Project project, List<ProcessDetailVO> processSchedule) {
        ProcessDetailVO detail = new ProcessDetailVO();
        detail.setProcessName(getProcessNode().getName());
        processSchedule.add(detail);
        // 根据项目是否通过立项批复(立项批复时间)来判断招标采购是否开始
        // 进行了立项批复且项目状态是待采购
        if (TO_BE_PURCHASED.eq(project.getStatus())) {
            detail.setStepStatus(StepStatusEnum.NOT_START);
        } else if (ON_PURCHASING.eq(project.getStatus())) {
            detail.setStepStatus(StepStatusEnum.ON_GOING);
        } else {
            List<Long> projectIds = projectService.allVersionProjectIds(project.getProjectCode());
            // 根据采购备案的时间获取
            ProjectStatusChange change = projectStatusChangeService.getOne(Wrappers.lambdaQuery(ProjectStatusChange.class)
                    .in(ProjectStatusChange::getProjectId, projectIds)
                    .eq(ProjectStatusChange::getEvent, ProjectStateChangeEvent.SUBMIT_OPERATION_PLAN)
                    .orderByDesc(ProjectStatusChange::getCreateOn)
                    .last(BizConst.LIMIT_1));
            if (change == null) {
                detail.setStepStatus(StepStatusEnum.NOT_START);
            } else {
                detail.setStepStatus(StepStatusEnum.COMPLETED);
                detail.setFinishTime(change.getCreateOn());
            }
        }
        detail.setProcessName(getProcessNode().getName());
    }

}
